;; routines to support bank switching
	.module banking
Banking__romSwitch = 0x2000

;; ====================
	.area	_HOME
	;; linker-generated entries in nonbanked jump table for all
	;; inter-bank calls;
	;; BC is filled with address in bank

Banking__switchTo1::
	LD	D,#1
	JR	Banking__switchGeneric

	;; routine for a banked call to bank 2
Banking__switchTo2::
	LD	D,#2
	JR	Banking__switchGeneric

	;; routine for a banked call to bank 3
Banking__switchTo3::
	LD	D,#3
;;  	JP	Banking__switchGeneric

	;; generic routine for banked calls
	;; BC is filled with address in bank
	;;  D is filled with bank number
Banking__switchGeneric::
	LD	A,(_Banking__currentBank)
	CP	A,D		; are we already in correct bank?
	JR	Z,1$

	;; process an inter-bank call
	LD	E,A
	LD	A,D		; do the bank switch
	LD	(_Banking__currentBank),A
	LD	(Banking__romSwitch),A
	LD	A,E

	;; push correct bank-return onto stack
	LD	D,#0
	SUB	A,#1
	ADD	A
	ADD	A		;  multiply by 4
	LD	E,A
	LD	HL,#Banking__returnTo1
	ADD	HL,DE

	POP	DE		; get real return address into DE
	PUSH	HL		; push Banking__returnToX onto stack

	PUSH	BC		; store address of banked routine
	LD	HL,#Banking__stackPtr	; put real return address to aux stack
	LD	A,(HL+)		; for later retrieval by Banking__returnToX
	LD	B,(HL)
	DEC	HL
	LD	C,A		;  BC points to stack entry
	LD	A,E
	LD	(BC),A
	INC	BC
	LD	A,D
	LD	(BC),A
	INC	BC		;  BC points to next free entry on stack
	LD	A,C
	LD	(HL+),A
	LD	(HL),B
	POP	BC		; retrieve address of banked routine
1$:
	LD	H,B
	LD	L,C
	JP	(HL)		; go to the routine

	;; routines which restore the calling bank after a banked call
Banking__returnTo1::
	LD	A,#1
	JR	Banking__returnGeneric

Banking__returnTo2::
	LD	A,#2
	JR	Banking__returnGeneric

Banking__returnTo3::
	LD	A,#3

Banking__returnGeneric::
	LD	(_Banking__currentBank),A
	LD	(Banking__romSwitch),A
	PUSH	HL		;  save possible function return value
	;; get return address from aux stack
	LD	HL,#Banking__stackPtr	; get pointer into banking stack
	LD	A,(HL+)
	LD	B,(HL)
	DEC	HL
	LD	C,A
	DEC	BC		; decrement pointer by two ("pop stack")
	DEC	BC
	LD	A,C
	LD	(HL+),A
	LD	(HL),B
	LD	L,C
	LD	H,B
	LD	A,(HL+)
	LD	B,(HL)
	LD	C,A
	;; restore registers and jump to calling routine
	POP	HL
	PUSH	BC
	RET

;==============================
	.area	_BSS
Banking__stack::
	.ds	0x50
Banking__stackPtr::
	.ds	2
_Banking__currentBank::
	.ds	1

 	.area	_GSINIT
	LD	A,#1
	LD	(_Banking__currentBank),A
	LD	DE,#Banking__stack	; store address of Banking__stack
	LD	HL,#Banking__stackPtr	; in Banking__stackPtr
	LD	A,E
	LD	(HL+),A
	LD	(HL),D