@ECHO OFF
REM -- DOS makefile for compiling and linking the Loderunner game

CALL ..\configure.bat
CALL localconfigure.bat

IF DEFINED TARGET_INCLUDE_ROOT_DIR GOTO :ELSE1
  ECHO ### ERROR: configuration file not adapted - please edit it first ###
  GOTO :ENDIF1
:ELSE1
  ECHO ### Loderunner: compiling game ###

  SETLOCAL

  REM --------------------------------------
  REM --- commands and other definitions ---
  REM --------------------------------------
  SET compileCommand=CALL %TOOLS_DIR%\sdcc-compile -o %TARGET_OBJECT_DIR%
  SET compileCommand=%compileCommand% -s %TARGET_SOURCE_DIR%

  REM ** optimize multiplications and use hand-optimized assembler code **
  SET SDCC_USER_DEFINES=-DmulDivAreAvoided -DuseOptimizedCode

  IF %TARGET_LANGUAGE_IS_ENGLISH%==false GOTO :ENDIF2
    SET SDCC_USER_DEFINES=%SDCC_USER_DEFINES% -DENGLISH
  :ENDIF2

  REM ------------------------------------------------------------

  REM -------------------
  REM --- compilation ---
  REM -------------------
  REM NOTE:
  REM modules with a --code 0 may be relocated to some bank 1..X
  REM modules without a --code go to bank 0 into the BASE segment

  %compileCommand% concretemap
  %compileCommand% concretemapanimation --code 0
  %compileCommand% concretemapview
  %compileCommand% concretemusic --code 0
  %compileCommand% game --code 0
  %compileCommand% gameobject --code 0
  %compileCommand% maplevel --code 0
  %compileCommand% opponent --code 0
  %compileCommand% player --code 0
  %compileCommand% preferences --code 0
  %compileCommand% rsc --code 0
  %compileCommand% rsc_data --code 0

  REM ---------------
  REM --- linking ---
  REM ---------------
  ECHO ### Loderunner: linking game ###

  SET objectList=%TARGET_OBJECT_DIR%\game.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\concretemap.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\concretemapanimation.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\concretemusic.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\concretemapview.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\gameobject.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\maplevel.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\opponent.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\player.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\preferences.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\rsc.o
  SET objectList=%objectList% %TARGET_OBJECT_DIR%\rsc_data.o

  REM set 4 ROM banks, 0 RAM banks and MBC type 2 as linking options
  SET SDCC_USER_LINK_OPTIONS=-yo4 -ya0 -yt2
  REM banking information is in bnk.txt
  SET bankAssignmentFile=%TARGET_OBJECT_DIR%\bnk.txt
  SET SDCC_USER_LINK_OPTIONS=%SDCC_USER_LINK_OPTIONS% -h%bankAssignmentFile%

  CALL %TOOLS_DIR%\sdcc-link %objectList%

  REM ---------------
  REM --- cleanup ---
  REM ---------------
  %RM_PROGRAM% *.map *.adb

  ENDLOCAL
:ENDIF1
